local AddonName, GGL = ...
--[[-----------------------------------------------------------------------------
LootAlertLine Widget

Events:

-------------------------------------------------------------------------------]]
local Type, Version = "GoldGoblinLootGUIWidget-LootAlertLine", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

local L = GGL.Locale

-- Lua APIs


-- WoW APIs
local CreateFrame = CreateFrame

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]
local ShowEditFrame, HideEditFrame
do
    local WIDTH_V, HEIGHT_V = 300, 330
    local WIDTH_M, HEIGHT_M = 0, 0

    local EditFrame

    local PROTO = {
        ["SetAlertLine"] = function(self, alertLine)
            self.alertLine = alertLine
        end,

        ["LoadRule"] = function(self, rule)
            self.rule = rule
            self:SetTitle(rule.name or UNKNOWN)
            self.name:SetText(rule.name)
            self.soundDD:SetValue(rule.sound or "None")
            self.valueOperatorDD:SetValue(rule.goldOperator or GGL.OPERATORS_DEFAULT)
            self.valueGold:SetText(rule.gold and rule.gold/10000 or 0)
            self.rarityOperatorDD:SetValue(rule.rarityOperator or GGL.OPERATORS_DEFAULT)
            self.rarityDD:SetValue((rule.rarity or 0) + 1)
            self.notifyDD:SetValue(rule.notify or 1)
            if self.isGlobal then
                self.isGlobal:SetValue(rule.isGlobal)
            end
            if self.priceSource then
                self.priceSource:SetValue(rule.priceSource or GGL.PriceSource:GetDefaultID())
                self.useVariations:SetValue(rule.useVariations)
            end
        end,
    }

    local function Name_OnEnterPressed(self, event, text)
        EditFrame.rule.name = text
        if EditFrame.alertLine then
            EditFrame.alertLine:Refresh()
        end
        EditFrame:SetTitle(text or UNKNOWN)
        self:SetText(text)
        self:ClearFocus()
    end

    local function IsGlobal_Check_OnValueChanged(self, event, state)
        EditFrame.rule.isGlobal = state
    end

    local function UseVariations_Check_OnValueChanged(self, event, state)
        EditFrame.rule.useVariations = state
    end

    -- ## valueGold
    local function ValueGoldOperator_OnValueChanged(self, event, text)
        EditFrame.rule.goldOperator = text
    end

    local function ValueGold_OnEnterPressed(self, event, text)
        if not text or text == "" or text == "0" then
            text = nil
        else
            text = tonumber(text)
            text = text and text*10000 or nil
        end
        EditFrame.rule.gold = text
        self:ClearFocus()
    end

    -- ## rarity
    local function RarityOperator_OnValueChanged(self, event, text)
        EditFrame.rule.rarityOperator = text
    end

    local function Rarity_OnValueChanged(self, event, text)
        EditFrame.rule.rarity = text - 1
    end

    -- ## price source
    local function PriceSource_OnValueChanged(self, event, text)
        EditFrame.rule.priceSource = text
    end

    -- ## sound
    local function Sound_OnValueChanged(self, event, text)
        EditFrame.rule.sound = text
        self:SetValue(text)
    end

    -- ## notify
    local function Notify_OnValueChanged(self, event, text)
        EditFrame.rule.notify = text
        self:SetValue(text)
    end

    local function CreateEditFrame()
        local window = AceGUI:Create("Window")

        local frame = window.frame
        local content = window.content

        -- some frame settings
        frame:SetMinResize(WIDTH_V,HEIGHT_V)
        frame:SetMaxResize(WIDTH_V+WIDTH_M,HEIGHT_V+HEIGHT_M)
        frame:SetClampedToScreen(true)
        frame:SetToplevel(true)
        frame:SetFrameStrata("FULLSCREEN_DIALOG")
        --window settings
        window:SetWidth(WIDTH_V)
        window:SetHeight(HEIGHT_V)
        -- remove resizer lines
        window.line1:Hide()
        window.line2:Hide()

        local contentGroup = AceGUI:Create("SimpleGroup")
        contentGroup:SetFullWidth(true)
        contentGroup:SetLayout("List")
        window:AddChild(contentGroup)

        local name = AceGUI:Create("EditBox")
        name:SetFullWidth(true)
        name:SetLabel(L["Name"])
        name:SetCallback("OnEnterPressed", Name_OnEnterPressed)
        contentGroup:AddChild(name)
        window.name = name

        local rulesLine = AceGUI:Create("Heading")
        rulesLine:SetText(L["Rules"])
        rulesLine:SetFullWidth(true)
        contentGroup:AddChild(rulesLine)
        window.rulesLine = rulesLine

        local valueGroup = AceGUI:Create("SimpleGroup")
        valueGroup:SetFullWidth(true)
        valueGroup:SetLayout("Flow")
        contentGroup:AddChild(valueGroup)

        local valueOperatorDD = AceGUI:Create("Dropdown")
        valueOperatorDD:SetList(GGL.OPERATORS_LIST) -- [name] = value
        valueOperatorDD:SetLabel(L["Gold"])
        valueOperatorDD:SetCallback("OnValueChanged", ValueGoldOperator_OnValueChanged)
        valueOperatorDD:SetRelativeWidth(0.3)
        valueGroup:AddChild(valueOperatorDD)
        window.valueOperatorDD = valueOperatorDD

        local valueGold = AceGUI:Create("EditBox")
        valueGold:SetRelativeWidth(0.7)
        --valueGold:SetLabel(L["Gold"])
        valueGold:SetCallback("OnEnterPressed", ValueGold_OnEnterPressed)
        valueGold.editbox:SetNumeric(true)
        valueGroup:AddChild(valueGold)
        window.valueGold = valueGold

        local rarityGroup = AceGUI:Create("SimpleGroup")
        rarityGroup:SetFullWidth(true)
        rarityGroup:SetLayout("Flow")
        contentGroup:AddChild(rarityGroup)

        local rarityOperatorDD = AceGUI:Create("Dropdown")
        rarityOperatorDD:SetList(GGL.OPERATORS_LIST) -- [name] = value
        rarityOperatorDD:SetLabel(L["Rarity"])
        rarityOperatorDD:SetCallback("OnValueChanged", RarityOperator_OnValueChanged)
        rarityOperatorDD:SetRelativeWidth(0.3)
        rarityGroup:AddChild(rarityOperatorDD)
        window.rarityOperatorDD = rarityOperatorDD

        local rarityList = {}
        for i = 0, #GGL.ITEM_QUALITY_INFO do
            rarityList[i+1] = GGL.ITEM_QUALITY_INFO[i].coloredDesc
        end
        local rarityDD = AceGUI:Create("Dropdown")
        rarityDD:SetList(rarityList) -- [name] = value
        rarityDD:SetCallback("OnValueChanged", Rarity_OnValueChanged)
        rarityDD:SetRelativeWidth(0.7)
        rarityGroup:AddChild(rarityDD)
        window.rarityDD = rarityDD

        local isGlobalGroup = AceGUI:Create("SimpleGroup")
        isGlobalGroup:SetFullWidth(true)
        isGlobalGroup:SetLayout("Flow")
        contentGroup:AddChild(isGlobalGroup)
        window.isGlobalGroup = isGlobalGroup

        local isGlobal = AceGUI:Create("CheckBox")
        isGlobal:SetFullWidth(true)
        isGlobal:SetLabel(L["Enable for all drops (Host only)"])
        isGlobal:SetCallback("OnValueChanged", IsGlobal_Check_OnValueChanged)
        GGL.GUI:AddToolTipForAceGUIWidget(isGlobal, L["Enable for all drops (Host only)"], L["This also includes items from other players."])
        isGlobalGroup:AddChild(isGlobal)
        window.isGlobal = isGlobal

        local alertsLine = AceGUI:Create("Heading")
        alertsLine:SetText(L["Alerts"])
        alertsLine:SetFullWidth(true)
        contentGroup:AddChild(alertsLine)
        window.alertsLine = alertsLine

        local soundDD = AceGUI:Create("LSM30_Sound")
        soundDD:SetFullWidth(true)
        soundDD:SetLabel(L["Sound"])
        soundDD:SetCallback("OnValueChanged", Sound_OnValueChanged)
        soundDD:SetList() -- load default LSM list
        contentGroup:AddChild(soundDD)
        window.soundDD = soundDD

        local notifyList = {}
        for i = 1, #GGL.LootAlert.NOTIFY_TYPES do
            notifyList[i] = GGL.LootAlert.NOTIFY_TYPES[i].name
        end
        local notifyDD = AceGUI:Create("Dropdown")
        notifyDD:SetFullWidth(true)
        notifyDD:SetLabel(L["Notify"])
        notifyDD:SetList(notifyList)
        notifyDD:SetCallback("OnValueChanged", Notify_OnValueChanged)
        contentGroup:AddChild(notifyDD)
        window.notifyDD = notifyDD


        for k, v in pairs(PROTO) do
            window[k] = v
        end

        EditFrame = window
    end

    function ShowEditFrame(self)
        if not self then return end
        if not EditFrame then
            CreateEditFrame()
        end

        local db = self.dbRule or {}
        EditFrame:SetAlertLine(self)

        -- for global we need the priceSource dropdown
        if self.isGlobalRule and not EditFrame.priceSource then
            -- release isGlobal checkbox
            EditFrame.isGlobalGroup:ReleaseChildren()
            EditFrame.isGlobal = nil

            -- add price source dropdown
            local priceSourceFrame = AceGUI:Create("Dropdown")
            local list, sortList = GGL.PriceSource:GetDropDownListData()
            priceSourceFrame:SetList(list, sortList)
            priceSourceFrame:SetLabel(L["Price Source"])
            priceSourceFrame:SetCallback("OnValueChanged", PriceSource_OnValueChanged)
            priceSourceFrame:SetFullWidth(true)
            EditFrame.isGlobalGroup:AddChild(priceSourceFrame)
            EditFrame.priceSource = priceSourceFrame

            -- add isGlobal checkbox
            local useVariations = AceGUI:Create("CheckBox")
            useVariations:SetFullWidth(true)
            useVariations:SetLabel(L["Variations"])
            useVariations:SetCallback("OnValueChanged", UseVariations_Check_OnValueChanged)
            useVariations:SetFullWidth(true)
            EditFrame.isGlobalGroup:AddChild(useVariations)
            EditFrame.useVariations = useVariations

            EditFrame:SetHeight(HEIGHT_V+15)
        elseif not self.isGlobalRule and EditFrame.priceSource then
            -- release price source dropdown
            EditFrame.isGlobalGroup:ReleaseChildren()
            EditFrame.priceSource = nil
            EditFrame.useVariations = nil

            -- add isGlobal checkbox
            local isGlobal = AceGUI:Create("CheckBox")
            isGlobal:SetFullWidth(true)
            isGlobal:SetLabel(L["Enable for all drops (Host only)"])
            isGlobal:SetCallback("OnValueChanged", IsGlobal_Check_OnValueChanged)
            GGL.GUI:AddToolTipForAceGUIWidget(isGlobal, L["Enable for all drops (Host only)"], L["This also includes items from other players."])
            EditFrame.isGlobalGroup:AddChild(isGlobal)
            EditFrame.isGlobal = isGlobal

            EditFrame:SetHeight(HEIGHT_V)
        end

        EditFrame:LoadRule(db)

        EditFrame:Show()
        AceGUI:SetFocus(EditFrame)
        EditFrame.frame:Raise()
    end

    function HideEditFrame(self)
        if EditFrame then
            EditFrame.frame:Hide()
        end
    end
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
    ["OnAcquire"] = function(self)
        self:SetRule()
        self:SetPosition()
        self:SetGlobal()
    end,

    ["OnRelease"] = function(self)
        HideEditFrame()
        self.frame:Hide()
    end,

    ["OnWidthSet"] = function(self, width)

    end,

    ["ShowEditFrame"] = function(self)
        if not self.dbRule then return end
        ShowEditFrame(self)
    end,

    ["SetRule"] = function(self, dbEntry)
        self.dbRule = dbEntry
        self:Refresh()
    end,

    ["SetGlobal"] = function(self, state)
        self.isGlobalRule = state
    end,

    ["SetPosition"] = function(self, position, numEntrys)
        self.position, self.numEntrys = position, numEntrys
        if position and numEntrys then
            -- downButton
            if position == numEntrys then
                self.downButton:Disable()
            else
                self.downButton:Enable()
            end

            -- upButton
            if position == 1 then
                self.upButton:Disable()
            else
                self.upButton:Enable()
            end
        else
            self.upButton:Disable()
            self.downButton:Disable()
        end
    end,

    ["SetEnableState"] = function(self, state)
        if not self.dbRule then return end
        if state then
            self.dbRule.disabled = nil
        else
            self.dbRule.disabled = true
        end
    end,

    ["SetBreakState"] = function(self, state)
        if not self.dbRule then return end
        if state then
            self.dbRule.noBreak = nil
        else
            self.dbRule.noBreak = true
        end
    end,

    ["Refresh"] = function(self)
        if self.dbRule then
            self.textName:SetText(self.dbRule.name or "")
            self.enableCheck:SetValue(not self.dbRule.disabled)
            self.breakCheck:SetValue(not self.dbRule.noBreak)
        else
            self.textName:SetText("")
            self.enableCheck:SetValue()
            self.breakCheck:SetValue()
        end
    end,
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function ButtonSetText(self, text)
    self.buttonObj:SetText(text)
end

local function CheckButtonSetText(self, text)
    self.text:SetText(text)
    self:SetWidth(self.text:GetStringWidth() + 10 + (self.frame:GetHeight() * 2))
end

local function Button_OnClick(self, ...)
    if self.type == "up" then
        self.obj:Fire("UpOnClick", ...)
    elseif self.type == "down" then
        self.obj:Fire("DownOnClick", ...)
    elseif self.type == "delete" then
        self.obj:Fire("DeleteOnClick", ...)
    elseif self.type == "edit" then
        self.obj:ShowEditFrame()
        self.obj:Fire("EditOnClick", ...)
    end
end

local function CheckButton_OnValueChanged(self, event, state)
    if self.type == "enable" then
        self.obj:SetEnableState(state)
        self.obj:Fire("OnEnableStateChanged", state)
    elseif self.type == "break" then
        self.obj:SetBreakState(state)
        self.obj:Fire("OnBreakStateChanged", state)
    end
end

local function NewButton(widget, content, buttonType)
    local buttonObj = AceGUI:Create("Button")
    buttonObj:SetAutoWidth(true)
    buttonObj:SetCallback("OnClick", Button_OnClick)

    local button = buttonObj.frame
    button:ClearAllPoints()
    button:SetParent(content)
    button:Show()

    button._SetText = button.SetText
    button.SetText = ButtonSetText
    button.buttonObj = buttonObj
    buttonObj.type = buttonType
    buttonObj.obj = widget

    return button
end

local function NewCheckButton(widget, content, buttonType)
    local checkButton = AceGUI:Create("CheckBox")
    checkButton.frame:ClearAllPoints()
    checkButton.frame:SetParent(content)
    checkButton.frame:Show()
    checkButton.type = buttonType
    checkButton.obj = widget

    checkButton:SetCallback("OnValueChanged", CheckButton_OnValueChanged)

    checkButton.SetLabel = CheckButtonSetText

    return checkButton
end

local TT_OPERATOR_F = "%s |cffffffff%s|r"
local function NameFrame_OnEnter(self)
    if not self.obj.dbRule then return end
    local db = self.obj.dbRule
    local tooltip = _G.GameTooltip
    tooltip:ClearLines()
    tooltip:SetOwner(UIParent,"ANCHOR_CURSOR")
    tooltip:SetText(db.name or UNKNOWN)
    tooltip:AddDoubleLine(L["Gold"],    format(TT_OPERATOR_F, GGL.OPERATORS_LIST[db.goldOperator or GGL.OPERATORS_DEFAULT], GGL.GetMoneyTextShort(db.gold or 0)))
    tooltip:AddDoubleLine(L["Rarity"],  format(TT_OPERATOR_F, GGL.OPERATORS_LIST[db.rarityOperator or GGL.OPERATORS_DEFAULT], GGL.ITEM_QUALITY_INFO[db.rarity or 0].coloredDesc))
    tooltip:AddDoubleLine(L["Sound"], db.sound or L["None"], nil, nil, nil, 1, 1, 1)
    tooltip:AddDoubleLine(L["Notify"], GGL.LootAlert.NOTIFY_TYPES[db.notify or 1].name or L["None"], nil, nil, nil, 1, 1, 1)
    if self.obj.isGlobalRule then
        tooltip:AddDoubleLine(L["Price Source"], GGL.PriceSource:GetName(db.priceSource) or UNKNOWN, nil, nil, nil, 1, 1, 1)
    end
    tooltip:Show()
end

local function NameFrame_OnLeave(self)
    _G.GameTooltip:Hide()
end

local function Constructor()
    local CreateLine = GGL.GUI.CreateLine

    local widget = {}
    widget.type  = Type

    local frame = GGL.GUI:CreateBorderedFrame(UIParent)
    widget.frame = frame

    local content = CreateFrame("Frame", nil, frame)
    content:SetPoint("TOPLEFT", frame, 5, -5)
    content:SetPoint("BOTTOMRIGHT", frame, -5, 5)
    content.obj = widget
    widget.content = content

    local upButton = NewButton(widget, content, "up")
    upButton:SetPoint("TOPRIGHT", content, -2, -2)
    upButton:SetText(L["Up"])
    widget.upButton = upButton

    local downButton = NewButton(widget, content, "down")
    downButton:SetPoint("TOPRIGHT", upButton, "BOTTOMRIGHT", 0, -2)
    downButton:SetText(L["Down"])
    widget.downButton = downButton

    local nameFrame = CreateFrame("Frame", nil, content)
    nameFrame:SetScript("OnEnter", NameFrame_OnEnter)
    nameFrame:SetScript("OnLeave", NameFrame_OnLeave)
    nameFrame:SetPoint("TOPRIGHT", upButton, "TOPLEFT", -2, 0)
    nameFrame:SetPoint("BOTTOMRIGHT", upButton, "BOTTOMLEFT", -2, 0)
    nameFrame:SetPoint("LEFT", content, "LEFT", 2, 0)
    nameFrame:SetHeight(12)
    nameFrame.obj = widget
    widget.nameFrame = nameFrame

    local textName = nameFrame:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
    textName:SetAllPoints(nameFrame)
    textName:SetJustifyH("LEFT")
    textName:SetJustifyV("MIDDLE")
    textName:SetWordWrap(false)
    widget.textName = textName

    local deleteButton = NewButton(widget, content, "delete")
    deleteButton:SetPoint("TOPLEFT", nameFrame, "BOTTOMLEFT", 0, -2)
    deleteButton:SetText(DELETE)
    widget.deleteButton = deleteButton

    local editButton = NewButton(widget, content, "edit")
    editButton:SetPoint("LEFT", deleteButton, "RIGHT", 2, 0)
    editButton:SetText(EDIT)
    widget.editButton = editButton

    local enableCheck = NewCheckButton(widget, content, "enable")
    enableCheck:SetLabel(ENABLE)
    GGL.GUI:AddToolTipForAceGUIWidget(enableCheck, ENABLE, L["Enable alert rule."])
    enableCheck.frame:SetPoint("LEFT", editButton, "RIGHT", 2, 0)
    widget.enableCheck = enableCheck

    local breakCheck = NewCheckButton(widget, content, "break")
    breakCheck:SetLabel(L["Break"])
    GGL.GUI:AddToolTipForAceGUIWidget(breakCheck, L["Break"], L["Stop checking the following rules when executed."])
    breakCheck.frame:SetPoint("RIGHT", downButton, "LEFT", -2, 0)
    widget.breakCheck = breakCheck


    frame:SetHeight(deleteButton:GetHeight()*2 + 16)

    -- methodes
    for method, func in pairs(methods) do
        widget[method] = func
    end

    return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
