local AddonName, GGL = ...
local Templates = GGL.Templates

local SourceFilter, SourceFilterHelper = Templates:Register("LootSourceFilter")

-- lua
local pairs = pairs
local bit_band = bit.band

-- wow

-- ##################
-- local
-- ##################
local MAIN_SOURCE_FLAG, SUB_SOURCE_FLAG = GGL.MAIN_SOURCE_FLAG, GGL.SUB_SOURCE_FLAG
local LOOT_SOURCE_TYPES, LOOT_SOURCE_IDS = GGL.LOOT_SOURCE_TYPES, GGL.LOOT_SOURCE_IDS

-- ##################
-- static
-- ##################
local Static = {}
Templates.SourceFilter = Static

local function CreateFilterDb(settings)
    if settings == true then
        return true
    end
    local db = {}
    for lootSourceTypeSID, lootSourceTypeS in pairs(settings) do
        local lootType = LOOT_SOURCE_IDS[lootSourceTypeSID]
        if lootType then
            if not db[lootType.mainSourceFilter] then db[lootType.mainSourceFilter] = {} end

            local usedDB
            for mainSourceName, mainSourceFlag in pairs(MAIN_SOURCE_FLAG) do
                if bit_band(lootType.mainSourceFilter, mainSourceFlag) ~= 0 then
                    db[mainSourceFlag] = db[mainSourceFlag] or {}
                    usedDB = db[mainSourceFlag]
                else
                    usedDB = nil
                end

                if usedDB then
                    for subSourceName, subSourceFlag in pairs(SUB_SOURCE_FLAG) do
                        if subSourceFlag == lootType.subSourceFilter then
                            usedDB[lootType.subSourceFilter] = true
                            break -- single one
                        elseif bit_band(lootType.subSourceFilter, subSourceFlag) ~= 0 then
                            usedDB[subSourceFlag] = true
                        end
                    end
                end
            end
        end
    end

    return db
end

-- ##################
-- template
-- ##################
function SourceFilter:Constructor(setKey, isSettingString)
    if isSettingString then
        self:LoadSetSettings(setKey)
    else
        self:LoadSet(setKey)
    end
end


function SourceFilter:LoadSet(setKey)
    local db = GGL.db.Config.Sets[GGL.SET_ID_LIST.LootSource]
    self.setKey = setKey
    if db and db[setKey] then
        local pass, db = GGL:Deserialize(db[setKey].setting)
        self:SetFilterDB(db)
    else
        self:SetFilterDB()
    end
end

function SourceFilter:LoadSetSettings(setting)
    if setting and setting ~= true then
        local pass, db = GGL:Deserialize(setting)
        self:SetFilterDB(db)
    else
        self:SetFilterDB(setting)
    end
end

function SourceFilter:SetFilterDB(db)
    if db then
        self.db = CreateFilterDb(db)
    else
        self.db = {}
    end
end

-- only support a combo of both
function SourceFilter:IsValidSource(mainSourceFlag, subSourceFlag)
    if not mainSourceFlag then return end

    subSourceFlag = subSourceFlag or 0
    local db = self.db

    if db ~= true and db[mainSourceFlag] and db[mainSourceFlag][subSourceFlag] then
        return false
    end

    return true
end